import Source from 'ol/source/Source.js';
import { WarpedMap, WarpedMapList } from '@allmaps/render';
import type { TransformationType } from '@allmaps/transform';
import type { Ring } from '@allmaps/types';
import type { Extent } from 'ol/extent';
/**
 * WarpedMapSource class. Together with a [WarpedMapLayer](#warpedmaplayer), this class
 * renders a warped map on an OpenLayers map.
 * @class WarpedMapSource
 * @extends import("ol/source/Source.js")
 */
export declare class WarpedMapSource extends Source {
    warpedMapList: WarpedMapList;
    /**
     * Creates a WarpedMapSource instance
     *
     * @constructor
     * @param {Cache} [imageInfoCache] - image info cache
     */
    constructor(imageInfoCache?: Cache);
    /**
     * Adds a [Georeference Annotation](https://iiif.io/api/extension/georef/).
     * @param {any} annotation - Georeference Annotation
     * @returns {Promise<(string | Error)[]>} - the map IDs of the maps that were added, or an error per map
     */
    addGeoreferenceAnnotation(annotation: unknown): Promise<(string | Error)[]>;
    /**
     * Removes a [Georeference Annotation](https://iiif.io/api/extension/georef/).
     * @param {any} annotation - Georeference Annotation
     * @returns {Promise<(string | Error)[]>} - the map IDs of the maps that were removed, or an error per map
     */
    removeGeoreferenceAnnotation(annotation: unknown): Promise<(string | Error)[]>;
    /**
     * Adds a [Georeference Annotation](https://iiif.io/api/extension/georef/) by URL.
     * @param {string} annotationUrl - Georeference Annotation
     * @returns {Promise<(string | Error)[]>} - the map IDs of the maps that were added, or an error per map
     */
    addGeoreferenceAnnotationByUrl(annotationUrl: string): Promise<(string | Error)[]>;
    /**
     * Removes a [Georeference Annotation](https://iiif.io/api/extension/georef/) by URL.
     * @param {string} annotationUrl - Georeference Annotation
     * @returns {Promise<(string | Error)[]>} - the map IDs of the maps that were removed, or an error per map
     */
    removeGeoreferenceAnnotationByUrl(annotationUrl: string): Promise<(string | Error)[]>;
    /**
     * Adds a Georeferenced map.
     * @param {unknown} georeferencedMap - Georeferenced map
     * @returns {Promise<(string | Error)>} - the map ID of the map that was added, or an error
     */
    addGeoreferencedMap(georeferencedMap: unknown): Promise<string | Error>;
    /**
     * Removes a Georeferenced map.
     * @param {unknown} georeferencedMap - Georeferenced map
     * @returns {Promise<(string | Error)>} - the map ID of the map that was remvoed, or an error
     */
    removeGeoreferencedMap(georeferencedMap: unknown): Promise<string | Error>;
    /**
     * Returns the WarpedMapList object that contains a list of the warped maps of all loaded maps
     * @returns {WarpedMapList} the warped map list
     */
    getWarpedMapList(): WarpedMapList;
    /**
     * Returns a single map's warped map
     * @param {string} mapId - ID of the map
     * @returns {WarpedMap | undefined} the warped map
     */
    getWarpedMap(mapId: string): WarpedMap | undefined;
    /**
     * Make a single map visible
     * @param {string} mapId - ID of the map
     */
    showMap(mapId: string): void;
    /**
     * Make multiple maps visible
     * @param {Iterable<string>} mapIds - IDs of the maps
     */
    showMaps(mapIds: Iterable<string>): void;
    /**
     * Make a single map invisible
     * @param {string} mapId - ID of the map
     */
    hideMap(mapId: string): void;
    /**
     * Make multiple maps invisible
     * @param {Iterable<string>} mapIds - IDs of the maps
     */
    hideMaps(mapIds: Iterable<string>): void;
    /**
     * Returns the visibility of a single map
     * @returns {boolean | undefined} - whether the map is visible
     */
    isMapVisible(mapId: string): boolean | undefined;
    /**
     * Sets the resource mask of a single map
     * @param {string} mapId - ID of the map
     * @param {Ring} resourceMask - new resource mask
     */
    setMapResourceMask(mapId: string, resourceMask: Ring): void;
    /**
     * Sets the transformation type of multiple maps
     * @param {Iterable<string>} mapIds - IDs of the maps
     * @param {TransformationType} transformation - new transformation type
     */
    setMapsTransformationType(mapIds: Iterable<string>, transformation: TransformationType): void;
    /**
     * Return the bounding box of all visible maps in the layer (inside or outside of the Viewport), in longitude/latitude coordinates.
     * @returns {Bbox | undefined} - Bounding box of all warped maps
     */
    getLonLatExtent(): Extent | undefined;
    /**
     * Return the bounding box of all visible maps in the layer (inside or outside of the Viewport), in projected coordinates.
     * @returns {Bbox | undefined} - bounding box of all warped maps
     */
    getExtent(): Extent | undefined;
    /**
     * Bring maps to front
     * @param {Iterable<string>} mapIds - IDs of the maps
     */
    bringMapsToFront(mapIds: Iterable<string>): void;
    /**
     * Send maps to back
     * @param {Iterable<string>} mapIds - IDs of the maps
     */
    sendMapsToBack(mapIds: string[]): void;
    /**
     * Bring maps forward
     * @param {Iterable<string>} mapIds - IDs of the maps
     */
    bringMapsForward(mapIds: Iterable<string>): void;
    /**
     * Send maps backward
     * @param {Iterable<string>} mapIds - IDs of the maps
     */
    sendMapsBackward(mapIds: Iterable<string>): void;
    /**
     * Returns the z-index of a single map
     * @param {string} mapId - ID of the warped map
     * @returns {number | undefined} - z-index of the warped map
     */
    getMapZIndex(mapId: string): number | undefined;
    /**
     * Sets the image info Cache of the WarpedMapList
     * @param {Cache} cache - the image info cache
     */
    setImageInfoCache(cache: Cache): void;
    /**
     * Clears the source, removes all maps
     */
    clear(): void;
}
