import Source from 'ol/source/Source.js';
import { WarpedMapList } from '@allmaps/render';
/**
 * WarpedMapSource class. Together with a [WarpedMapLayer](#warpedmaplayer), this class
 * renders a warped map on an OpenLayers map.
 * @class WarpedMapSource
 * @extends import("ol/source/Source.js")
 */
export class WarpedMapSource extends Source {
    /**
     * Creates a WarpedMapSource instance
     *
     * @constructor
     * @param {Cache} [imageInfoCache] - image info cache
     */
    constructor(imageInfoCache) {
        super({
            interpolate: true,
            projection: undefined,
            state: 'ready',
            wrapX: true
        });
        this.warpedMapList = new WarpedMapList(imageInfoCache);
    }
    /**
     * Adds a [Georeference Annotation](https://iiif.io/api/extension/georef/).
     * @param {any} annotation - Georeference Annotation
     * @returns {Promise<(string | Error)[]>} - the map IDs of the maps that were added, or an error per map
     */
    async addGeoreferenceAnnotation(annotation) {
        const results = await this.warpedMapList.addGeoreferenceAnnotation(annotation);
        this.changed();
        return results;
    }
    /**
     * Removes a [Georeference Annotation](https://iiif.io/api/extension/georef/).
     * @param {any} annotation - Georeference Annotation
     * @returns {Promise<(string | Error)[]>} - the map IDs of the maps that were removed, or an error per map
     */
    async removeGeoreferenceAnnotation(annotation) {
        const results = await this.warpedMapList.removeGeoreferenceAnnotation(annotation);
        this.changed();
        return results;
    }
    /**
     * Adds a [Georeference Annotation](https://iiif.io/api/extension/georef/) by URL.
     * @param {string} annotationUrl - Georeference Annotation
     * @returns {Promise<(string | Error)[]>} - the map IDs of the maps that were added, or an error per map
     */
    async addGeoreferenceAnnotationByUrl(annotationUrl) {
        const annotation = await fetch(annotationUrl).then((response) => response.json());
        const results = this.addGeoreferenceAnnotation(annotation);
        return results;
    }
    /**
     * Removes a [Georeference Annotation](https://iiif.io/api/extension/georef/) by URL.
     * @param {string} annotationUrl - Georeference Annotation
     * @returns {Promise<(string | Error)[]>} - the map IDs of the maps that were removed, or an error per map
     */
    async removeGeoreferenceAnnotationByUrl(annotationUrl) {
        const annotation = await fetch(annotationUrl).then((response) => response.json());
        const results = this.removeGeoreferenceAnnotation(annotation);
        return results;
    }
    /**
     * Adds a Georeferenced map.
     * @param {unknown} georeferencedMap - Georeferenced map
     * @returns {Promise<(string | Error)>} - the map ID of the map that was added, or an error
     */
    async addGeoreferencedMap(georeferencedMap) {
        const result = this.warpedMapList.addGeoreferencedMap(georeferencedMap);
        this.changed();
        return result;
    }
    /**
     * Removes a Georeferenced map.
     * @param {unknown} georeferencedMap - Georeferenced map
     * @returns {Promise<(string | Error)>} - the map ID of the map that was remvoed, or an error
     */
    async removeGeoreferencedMap(georeferencedMap) {
        const result = this.warpedMapList.removeGeoreferencedMap(georeferencedMap);
        this.changed();
        return result;
    }
    /**
     * Returns the WarpedMapList object that contains a list of the warped maps of all loaded maps
     * @returns {WarpedMapList} the warped map list
     */
    getWarpedMapList() {
        return this.warpedMapList;
    }
    /**
     * Returns a single map's warped map
     * @param {string} mapId - ID of the map
     * @returns {WarpedMap | undefined} the warped map
     */
    getWarpedMap(mapId) {
        return this.warpedMapList.getWarpedMap(mapId);
    }
    /**
     * Make a single map visible
     * @param {string} mapId - ID of the map
     */
    showMap(mapId) {
        this.warpedMapList.showMaps([mapId]);
        this.changed();
    }
    /**
     * Make multiple maps visible
     * @param {Iterable<string>} mapIds - IDs of the maps
     */
    showMaps(mapIds) {
        this.warpedMapList.showMaps(mapIds);
        this.changed();
    }
    /**
     * Make a single map invisible
     * @param {string} mapId - ID of the map
     */
    hideMap(mapId) {
        this.warpedMapList.hideMaps([mapId]);
        this.changed();
    }
    /**
     * Make multiple maps invisible
     * @param {Iterable<string>} mapIds - IDs of the maps
     */
    hideMaps(mapIds) {
        this.warpedMapList.hideMaps(mapIds);
        this.changed();
    }
    /**
     * Returns the visibility of a single map
     * @returns {boolean | undefined} - whether the map is visible
     */
    isMapVisible(mapId) {
        const warpedMap = this.warpedMapList.getWarpedMap(mapId);
        return warpedMap?.visible;
    }
    /**
     * Sets the resource mask of a single map
     * @param {string} mapId - ID of the map
     * @param {Ring} resourceMask - new resource mask
     */
    setMapResourceMask(mapId, resourceMask) {
        this.warpedMapList.setMapResourceMask(mapId, resourceMask);
        this.changed();
    }
    /**
     * Sets the transformation type of multiple maps
     * @param {Iterable<string>} mapIds - IDs of the maps
     * @param {TransformationType} transformation - new transformation type
     */
    setMapsTransformationType(mapIds, transformation) {
        this.warpedMapList.setMapsTransformationType(mapIds, transformation);
        this.changed();
    }
    /**
     * Return the bounding box of all visible maps in the layer (inside or outside of the Viewport), in longitude/latitude coordinates.
     * @returns {Bbox | undefined} - Bounding box of all warped maps
     */
    getLonLatExtent() {
        return this.warpedMapList.getBbox();
    }
    /**
     * Return the bounding box of all visible maps in the layer (inside or outside of the Viewport), in projected coordinates.
     * @returns {Bbox | undefined} - bounding box of all warped maps
     */
    getExtent() {
        return this.warpedMapList.getProjectedBbox();
    }
    /**
     * Bring maps to front
     * @param {Iterable<string>} mapIds - IDs of the maps
     */
    bringMapsToFront(mapIds) {
        this.warpedMapList.bringMapsToFront(mapIds);
        this.changed();
    }
    /**
     * Send maps to back
     * @param {Iterable<string>} mapIds - IDs of the maps
     */
    sendMapsToBack(mapIds) {
        this.warpedMapList.sendMapsToBack(mapIds);
        this.changed();
    }
    /**
     * Bring maps forward
     * @param {Iterable<string>} mapIds - IDs of the maps
     */
    bringMapsForward(mapIds) {
        this.warpedMapList.bringMapsForward(mapIds);
        this.changed();
    }
    /**
     * Send maps backward
     * @param {Iterable<string>} mapIds - IDs of the maps
     */
    sendMapsBackward(mapIds) {
        this.warpedMapList.sendMapsBackward(mapIds);
        this.changed();
    }
    /**
     * Returns the z-index of a single map
     * @param {string} mapId - ID of the warped map
     * @returns {number | undefined} - z-index of the warped map
     */
    getMapZIndex(mapId) {
        return this.warpedMapList.getMapZIndex(mapId);
    }
    /**
     * Sets the image info Cache of the WarpedMapList
     * @param {Cache} cache - the image info cache
     */
    setImageInfoCache(cache) {
        this.warpedMapList.setImageInfoCache(cache);
    }
    /**
     * Clears the source, removes all maps
     */
    clear() {
        this.warpedMapList.clear();
        this.changed();
    }
}
